(function() {
  var child_npm, fs, http, httpProxy, parent_npm, program, proxy, rest, server, strip_url_for_npm_vhost, targetParser;

  fs = require('fs');

  http = require('http');

  program = require('commander');

  httpProxy = require('http-proxy');

  rest = require('restler');

  targetParser = function(target) {
    var host, port, _ref;
    _ref = target.split(':'), host = _ref[0], port = _ref[1];
    return {
      host: host,
      port: parseInt(port, 10) || 80
    };
  };

  program.version('0.0.1').option('-t, --target [localhost:8080]', 'hostname and port to listen on', targetParser, {
    host: 'localhost',
    port: 8080
  }).option('-c, --child-registry-target [localhost:5984]', 'NPM registry to push packages and pull from first', targetParser, {
    host: 'localhost',
    port: 5984
  }).option('-p, --parent-registry-target [registry.npmjs.org:80]', "NPM registry to pull packages from if the child doesn't have it", targetParser, {
    host: 'registry.npmjs.org',
    port: 80
  }).parse(process.argv);

  proxy = program.target;

  parent_npm = program.parentRegistryTarget;

  child_npm = program.childRegistryTarget;

  child_npm.check = function(req) {
    var url;
    url = ("http://" + this.host + ":" + this.port) + req.url;
    console.log("" + req.method + ":", url);
    req.headers['host'] = "" + this.host + ":" + this.port;
    return rest.get(url, {
      headers: req.headers
    });
  };

  strip_url_for_npm_vhost = function(url) {
    return url.replace('registry/_design/app/_rewrite/', '');
  };

  server = httpProxy.createServer(function(req, res, proxy) {
    var buffer, check, proxyOpts;
    buffer = httpProxy.buffer(req);
    if (req.method === 'GET') {
      check = child_npm.check(req);
      check.on('success', function(data) {
        var proxyOpts;
        console.log("INFO: Found " + req.url + " in child npm!");
        req.headers['host'] = child_npm.host;
        proxyOpts = child_npm;
        proxyOpts['buffer'] = buffer;
        return proxy.proxyRequest(req, res, proxyOpts);
      });
      return check.on('error', function(err) {
        var proxyOpts;
        console.error("ERROR: Can't find " + req.url + " in child npm");
        req.url = strip_url_for_npm_vhost(req.url);
        console.info("INFO: proxying to parent npm " + req.url);
        req.headers['host'] = parent_npm.host;
        proxyOpts = parent_npm;
        proxyOpts['buffer'] = buffer;
        return proxy.proxyRequest(req, res, proxyOpts);
      });
    } else {
      console.info("INFO: Updating " + req.url);
      req.headers['host'] = child_npm.host;
      proxyOpts = child_npm;
      proxyOpts['buffer'] = buffer;
      return proxy.proxyRequest(req, res, proxyOpts);
    }
  });

  server.on('error', function(err) {
    return console.error('ERROR: Server Error:', err);
  });

  server.listen(proxy.port, proxy.host, function() {
    return console.info("Proxying " + proxy.host + ":" + proxy.port);
  });

}).call(this);
